-- TODO
--
-- * Make the display (and tooltip) show KB/s (KiB/s?) when appropriate.
-- * For icon, use the popular "2 computers" icon instead of the current SysMon one.
-- * Adjust the average-over time, perhaps 60 seconds is more fitting? Needs testing
--   in a raid environment. We don't want the values jumping around like crazy, better
--   to show something more stable, even if it's a "lie".
--

-- How many seconds of data to use when calculating the average bytes per second.
local AVERAGE_OVER = 10

-- Locale strings
local L_TITLE = "Transfer stats"
local L_TIP = "Transfer statistics for the addon communication channel shown in bytes per second."
local L_AVERAGE = "Current average"
local L_AVERAGE_OVER = "Average over"
local L_SECONDS = "%d sec"
local L_HIGHEST_AVG = "Highest average"
local L_HIGHEST_PEAK = "Peak second"
local L_BS = "bps"
local L_BYTES = "%d bytes"
local L_DESC = "Bytes per second (10-sec average)"

local _G = getfenv(0)
local text = "%.1f"
local broker = LibStub("LibDataBroker-1.1")
-- Icon shamelessly borrowed from the Tango Project
local icon = "Interface\\AddOns\\Broker_CMA\\icon"
local CMA = broker:NewDataObject("Broker_CMA", {
	icon = icon,
	suffix = L_BS,
	label = L_DESC,
	type = "data source",
})
local f = CreateFrame("Frame")
local totalElapsed = 0
local lastSecond = 0
local highestAverage = 0
local highestPeak = 0

local r = setmetatable({}, {__add = function(t, n)
	table.insert(t, n)
	if #t > AVERAGE_OVER then table.remove(t, 1) end
	local x = 0
	for i = 1, #t do x = x + t[i] end
	return x
end})
f:SetScript("OnUpdate", function(self, elapsed)
	totalElapsed = totalElapsed + elapsed
	if totalElapsed >= 1 then
		local lastPeriod = r + lastSecond
		local average = lastPeriod / #r
		CMA.value = text:format(average)
		CMA.text = format("%s %s", CMA.value, CMA.suffix)

		-- Calculate highscores!		
		if lastSecond > highestPeak then highestPeak = lastSecond end
		-- But only calculate highest average if we've gathered enough data
		if #r == AVERAGE_OVER and average > highestAverage then highestAverage = average end

		lastSecond = 0
		totalElapsed = 0
	end
end)
f:SetScript("OnEvent", function(self, event, prefix, message, distribution, sender)
	-- 40 is a guesstimate message overhead stolen from ChatThrottleLib by Mikk.
	-- This "total message size" estimate is probably (hopefully) a bit bigger than it should be.
	local size = prefix:len() + message:len() + distribution:len() + sender:len() + 40
	lastSecond = lastSecond + size
end)
f:RegisterEvent("CHAT_MSG_ADDON")
f:Show()

function CMA.OnTooltipShow(tt)
	if not tt or not tt.AddLine or not tt.AddDoubleLine then return end
	tt:AddLine(L_TITLE)
	tt:AddDoubleLine(L_AVERAGE, format("%s %s", CMA.value, CMA.suffix), 1, 1, 1, 0, 1, 0)
	tt:AddDoubleLine(L_AVERAGE_OVER, L_SECONDS:format(AVERAGE_OVER), 1, 1, 1, 0, 1, 0)
	tt:AddDoubleLine(L_HIGHEST_AVG, ("%.1f %s"):format(highestAverage, L_BS), 1, 1, 1, 0, 1, 0)
	tt:AddDoubleLine(L_HIGHEST_PEAK, L_BYTES:format(highestPeak), 1, 1, 1, 0, 1, 0)
	tt:AddLine(" ")
	tt:AddLine(L_TIP, 0.2, 1, 0.2, 1)
end

-- For debugging
--_G.BrokerCMATable = r

